/*
   Note: portions of the original parser code were inspired by
   the feed reader software Rol which is copyrighted by
   
   Copyright (C) 2002 Jonathan Gordon <eru@unknown-days.com>
   
   The major part of this backend/parsing/storing code written by
   
   Copyright (C) 2003 Lars Lindner <lars.lindner@gmx.net>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <time.h>
#include <gconf/gconf.h>	// FIXME: remove
#include "support.h"
#include "conf.h"

#include "ns_dc.h"
#include "ns_slash.h"
#include "ns_content.h"

#include "backend.h"
#include "callbacks.h"	

#define	TIMESTRLEN	256

GtkTreeStore	*feedstore = NULL;
GtkTreeStore	*itemstore = NULL;

GtkTreeIter	*myfeedgroup = NULL;
GtkTreeIter	*availablefeedgroup = NULL;

/* used to lookup a feed pointer specified by a key */
GHashTable	*feeds = NULL;

/* to store the nsHanlder structs for all supported RDF namespace handlers */
GHashTable	*nslist = NULL;

extern GdkPixbuf	*unreadIcon;
extern GdkPixbuf	*readIcon;
extern GdkPixbuf	*availableIcon;
extern GdkPixbuf	*unavailableIcon;


/* the encoding all item descriptions are converted to for display with libGtkHTML */
static gchar	*standard_encoding = { "UTF-8" };

/* pointer to data of a newly created feed */
static channelPtr	new_cp = NULL;

/* FIXME: due to bad newFeed() design we need a global variable to hold a
   pointer to an channel for update calls of newFeed()... */
static channelPtr	old_cp = NULL;

/* prototypes */
void setInFeedList(gchar *feedkey, gchar *feedname, gchar *feedurl, GdkPixbuf *state, channelPtr cp);
void addToFeedList(channelPtr cp);
void showFeed(gpointer cp);

/* ------------------------------------------------------------------------- */

guint hashFunction(gconstpointer key) {	return (guint)atoi((char *)key); }
gint feedsHashCompare(gconstpointer a, gconstpointer b) { return a-b; }

/* initializing function */
void initBackend() {
	if(NULL == feeds)
		feeds = g_hash_table_new(g_str_hash, g_str_equal);

	g_free(nslist);
	nslist = g_hash_table_new(g_str_hash, g_str_equal);
	
	/* register name space handlers */
	if(getBooleanConfValue(USE_DC))
		g_hash_table_insert(nslist, (gpointer)ns_dc_getNSPrefix(),
					    (gpointer)ns_dc_getNameSpaceHandler());
	if(getBooleanConfValue(USE_SLASH))
		g_hash_table_insert(nslist, (gpointer)ns_slash_getNSPrefix(), 
					    (gpointer)ns_slash_getNameSpaceHandler());
	if(getBooleanConfValue(USE_CONTENT))
		g_hash_table_insert(nslist, (gpointer)ns_content_getNSPrefix(),
					    (gpointer)ns_content_getNameSpaceHandler());
}

/* converts the string string encoded in from_encoding (which
   can be NULL) to to_encoding, frees the original string and 
   returns the result */
gchar * convertCharSet(gchar * from_encoding, gchar * to_encoding, gchar * string) {
	gint	bw, br;
	gchar	*new = NULL;
	
	// FIXME: is this thread safe?
	if(NULL == from_encoding)
		from_encoding = standard_encoding;
		
	if(NULL != string) {		
		new = g_convert(string, strlen(string), from_encoding, to_encoding, &br, &bw, NULL);
		
		if(NULL != new)
			g_free(string);
		else
			new = string;
	} else {	
		return g_strdup("");
	}

	return new;
}

gchar * convertToUTF8(gchar * from_encoding, gchar * string) { return convertCharSet(from_encoding, "UTF-8", string); }
gchar * convertToHTML(gchar * from_encoding, gchar * string) { return convertCharSet(from_encoding, "HTML", string); }

gchar * parseHTML(htmlDocPtr doc, htmlNodePtr cur, gchar *string) {
	gchar	*newstring;
	gchar	*oldstring;
	
	cur = cur->xmlChildrenNode;
	while (cur != NULL) {
		if(cur->content != NULL) {
			// g_print("%s >>>%s<<<\n", cur->name, cur->content);
			oldstring = newstring;
			newstring = g_strdup_printf("%s%s", newstring, cur->content);
			g_free(oldstring);
		}
		
		if(cur->xmlChildrenNode != NULL)
			newstring = parseHTML(doc, cur->xmlChildrenNode, newstring);
			
		cur = cur->next;		
	}
	
	return newstring;
}

/* converts strings containing any HTML stuff

   FIXME: still buggy, correctly converts entities and
   preserves encodings, but does loose text inside enclosing
   formatting tags like "<b>Hallo</b>" 
 */
gchar * unhtmlize(gchar * from_encoding, gchar *string) {
	htmlParserCtxtPtr	ctxt; 
	xmlDocPtr		pDoc;
	int			length;
	gchar			*newstring = NULL;
	
	/* only do something if there are entities */
	if(NULL == (strchr(string, '&')))
		return string;
	
	string = convertToUTF8(from_encoding, string);
		
	length = strlen(string);
	newstring = (gchar *)g_malloc(length + 1);
	memset(newstring, 0, length + 1);
	ctxt = htmlCreatePushParserCtxt(NULL, NULL, newstring, length, 0, (xmlCharEncoding)from_encoding);

        htmlParseChunk(ctxt, string, length, 0);
        htmlParseChunk(ctxt, string, 0, 1);
        pDoc = ctxt->myDoc;
	newstring = parseHTML(pDoc, xmlDocGetRootElement(pDoc), "");	
        htmlFreeParserCtxt(ctxt);
	
	g_free(string);
	
	return newstring;
}

/* method to parse standard tags for the channel element */
void parseChannel(channelPtr c, xmlDocPtr doc, xmlNodePtr cur) {
	gchar			*tmp = NULL;
	gchar			*encoding;
	parseChannelTagFunc	fp;
	nsHandler		*nsh;
	
	cur = cur->xmlChildrenNode;
	while (cur != NULL) {
	
		/* check namespace of this tag */
		if(NULL != cur->ns) {
			if (NULL != cur->ns->prefix) {
				if(NULL != (nsh = (nsHandler *)g_hash_table_lookup(nslist, (gpointer)cur->ns->prefix))) {	
					fp = nsh->parseChannelTag;
					if(NULL != fp)
						(*fp)(c, doc, cur);
					cur = cur->next;
					continue;
				} else {
					g_print("unsupported namespace \"%s\"\n", cur->ns->prefix);
				}
			}
		} 
		
		if(!xmlStrcmp(cur->name, (const xmlChar *)"title")) {
			c->title = g_strdup(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
			c->title = unhtmlize((gchar *)doc->encoding, c->title);
		}

		if(!xmlStrcmp(cur->name, (const xmlChar *)"description")) {
			tmp = g_strdup(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
			c->description = convertToHTML((gchar *)doc->encoding, tmp);
		}

		if(!xmlStrcmp(cur->name, (const xmlChar *)"link"))
			c->link = g_strdup(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));

		cur = cur->next;
	}
}

/* method to parse standard tags for each item element */
itemPtr parseItem(xmlDocPtr doc, xmlNodePtr cur) {
	gint			bw, br;
	gchar			*tmp2, *tmp = NULL;
	gchar			*entity, *entityend;
	int 			insize, outsize;
	gchar			*encoding;
	parseItemTagFunc	fp;
	nsHandler		*nsh;	
	itemPtr 		i = NULL;

	if(NULL == (i = (itemPtr) malloc(sizeof(item)))) {
		g_error("not enough memory!\n");
		return(NULL);
	}
	memset(i, 0, sizeof(item));
	i->nsinfos = g_hash_table_new(g_str_hash, g_str_equal);

	cur = cur->xmlChildrenNode;
	while (cur != NULL) {
		/* check namespace of this tag */
		if(NULL != cur->ns) {		
			if (NULL != cur->ns->prefix) {
				if(NULL != (nsh = (nsHandler *)g_hash_table_lookup(nslist, (gpointer)cur->ns->prefix))) {	
					fp = nsh->parseItemTag;
					if(NULL != fp)
						(*fp)(i, doc, cur);
					cur = cur->next;
					continue;						
				} else {
					g_print("unsupported namespace \"%s\"\n", cur->ns->prefix);
				}
			}
		}
		
		if (!xmlStrcmp(cur->name, (const xmlChar *)"title")) {
			i->title = g_strdup(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));	
			i->title = unhtmlize((gchar *)doc->encoding, i->title);
		}

		if (!xmlStrcmp(cur->name, (const xmlChar *)"description")) {
			tmp = g_strdup(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
			i->description = convertToHTML((gchar *)doc->encoding, tmp);
		}

		if (!xmlStrcmp(cur->name, (const xmlChar *)"link"))
			i->link = g_strdup(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));

		if (!xmlStrcmp(cur->name, (const xmlChar *)"image"))
			i->image = g_strdup(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));			

		cur = cur->next;
	}
	
	return(i);
}

void parseImage(xmlDocPtr doc, xmlNodePtr cur, channelPtr cp) {

	cur = cur->xmlChildrenNode;
	while (cur != NULL) {
		if (!xmlStrcmp(cur->name, (const xmlChar *)"url"))
			cp->image = g_strdup(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
			
		cur = cur->next;
	}
}

void updateFeed(gpointer cp) {
	channelPtr	c = (channelPtr)cp;
	
	// FIXME: implement incremental item lists
	print_status(g_strdup_printf("updating \"%s\"", getFeedTitle(cp)));

	old_cp = cp;
		
	// FIXME: free old feed memory	
	newFeed(c->feedkey, c->source, FALSE, FALSE);

	showFeed(cp);
	loadItems(cp);
	
	old_cp = NULL;
}

/* tries to read the feed and sets default title in new dialog */
int newFeed(gchar *feedkey, gchar *url, gboolean newflag, gboolean createflag) {
	xmlDocPtr 	doc;
	xmlNodePtr 	cur;
	time_t		t;
	itemPtr 	ip, lastip = NULL;
	channelPtr 	cp;
	gchar		*encoding;
	gchar		*timestr;
	gchar		*timeformat;
	short 		rdf = 0;
	int 		error = 0;
	
	/* variables used for XML to memory reading...
	void 		*ctx;
	char 		*contentType;
	char 		*buf = NULL;
	int 		readcount=0;
	int 		bufsize=0;
	int 		i;
	*/
	
	/* initialize channel structure */
	// FIXME channel should not be the uppermost structure!
	// better something like a feed structure
	if(newflag) {
		if(NULL == (cp = (channelPtr) malloc(sizeof(channel)))) {
			g_error("not enough memory!\n");
			return 1;
		}
		memset(cp, 0, sizeof(channel));
		cp->nsinfos = g_hash_table_new(g_str_hash, g_str_equal);		
	} else {
		cp = (channelPtr)old_cp;	/* reuse old cp for updates... */
		g_assert(NULL != cp);
	}
	
	// FIXME: free items on update

	if(NULL == feedkey) 
		cp->feedkey = getFreeFeedKey();
	else	
		cp->feedkey = feedkey;
		
	g_hash_table_insert(feeds, (gpointer)cp->feedkey, (gpointer)cp);
	
	cp->items = NULL;
	cp->available = FALSE;
	cp->source = g_strdup(url);
	new_cp = (gpointer)cp;
	
	while(1) {	
		g_print(_("reading from %s\n"),url);
		
/*		// test code to read XML into memory and maybe
		// adjust encodings (planned feature)...

		#define XML_BLOCK_SIZE	2048
		ctx = xmlNanoHTTPOpen(url, &contentType);
		do {
			bufsize += XML_BLOCK_SIZE;
			buf = (char *)g_realloc(buf, bufsize + XML_BLOCK_SIZE);
			readcount = xmlNanoHTTPRead(ctx, buf+bufsize-XML_BLOCK_SIZE, XML_BLOCK_SIZE);
			g_print("read %d bytes/%d\n", readcount,bufsize);
		} while(readcount>0);
		xmlNanoHTTPClose(ctx);
		
		for(i=0;i<100;i++) {
			g_print("%c", *(buf+i));
		}
		
		doc = xmlParseMemory(buf, bufsize);*/
		doc = xmlParseFile(url);

		if(NULL == doc) {
			print_status(g_strdup_printf(_("XML error wile reading feed! Feed \"%s\" could not be loaded!"),url));
			error = 1;
			break;
		}

		cur = xmlDocGetRootElement(doc);

		if(NULL == cur) {
			print_status(_("Empty document! Feed was not added!"));
			xmlFreeDoc(doc);
			error = 1;
			break;			
		}

		if (!xmlStrcmp(cur->name, (const xmlChar *)"rss")) {
			rdf = 0;
		} else if (!xmlStrcmp(cur->name, (const xmlChar *)"rdf") || 
                	   !xmlStrcmp(cur->name, (const xmlChar *)"RDF")) {
			rdf = 1;
		} else {
			print_status(_("Could not find RDF/RSS header! Feed was not added!"));
			xmlFreeDoc(doc);
			error = 1;
			break;			
		}

		cur = cur->xmlChildrenNode;
		while (cur && xmlIsBlankNode(cur)) {
			cur = cur->next;
		}
		if (cur == 0) {
			error = 1;
			break;
		}

		if (xmlStrcmp(cur->name, (const xmlChar *) "channel")) {
			print_status(_("Could not find channel element."));
			error = 1;
			break;
		}

		if (rdf == 0) {
			parseChannel(cp, doc, cur);
			cur = cur->xmlChildrenNode;
		} else if (rdf == 1) {
			while (cur != NULL) {
				if ((!xmlStrcmp(cur->name, (const xmlChar *) "channel"))) {			
					parseChannel(cp, doc, cur);
					break;
				}
			}
		} else {
			g_error("This shouldn't happen.\n");
			error = 1;
			break;
		}

		/* get receive time */
		if((time_t)-1 != time(&t)) {
			if(NULL != (timestr = (gchar *)g_malloc(TIMESTRLEN+1))) {
				if(NULL != (timeformat = getStringConfValue(TIME_FORMAT))) {
					strftime(timestr, TIMESTRLEN, (char *)timeformat, gmtime(&t));
					g_free(timeformat);
				}
			}
		}

		cp->time = timestr;
		cp->encoding = g_strdup(doc->encoding);
		cp->available = TRUE;

		/* parse channel contents */
		while (cur != NULL) {
			/* save link to channel image */
			if ((!xmlStrcmp(cur->name, (const xmlChar *) "image"))) {
				parseImage(doc, cur, cp);
			}

			/* collect channel items */
			if ((!xmlStrcmp(cur->name, (const xmlChar *) "item"))) {
				if(NULL != (ip = (itemPtr) parseItem(doc, cur))) {
					if(NULL == ip->time)
						ip->time = cp->time;
					ip->next = NULL;
					if(NULL == lastip)
						cp->items = ip;
					else
						lastip->next = ip;
					lastip = ip;				
				}
			}
			cur = cur->next;
		}

		xmlFreeDoc(doc);
		break;
	}

	if(createflag) {
		if(error) {
			g_print(_("Feed was not added!"));
		} else {
			addFeedToConfig(cp->feedkey, url);
		}
	}

	return error;
}

void removeFeed(gpointer cp) {
	GtkTreeIter	iter;
	gboolean	valid, found = 0;	
	gchar		*tmp_title = NULL;
	gchar		*tmp_url = NULL;
	GdkPixbuf	*tmp_state = NULL;
	channelPtr	tmp_cp;

	/* find feed in feedstore */
	valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(feedstore), &iter);
	while(valid) {
		gtk_tree_model_get(GTK_TREE_MODEL(feedstore), &iter,
				FS_TITLE, &tmp_title,
				FS_URL, &tmp_url,
				FS_CP, &tmp_cp,
		  	      -1);

		if((channelPtr)cp == tmp_cp)
			found = 1;		

		if(found) {
			/* remove row contents */
			gtk_tree_store_remove(feedstore, &iter);
		}
		
		g_free(tmp_title);
		g_free(tmp_url);
		
		if(found)
			break;
		
		valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(feedstore), &iter);
	}

	if(found) {
		// FIXME: free feed data structures...
		removeFeedFromConfig(((channelPtr)cp)->feedkey);
	} else {
		print_status(_("Internal error while deleting feed from backend!"));
	}
}

/* shows feed after loading on startup or creations of a feed */
void showFeed(gpointer cp) {
	channelPtr	c = (channelPtr)cp;

	setInFeedList(c->feedkey, 
		      (c->usertitle)?c->usertitle:c->title,
		      c->source,
		      (c->available)?availableIcon:unavailableIcon,
		      c);
}

void setFeedTitle(gpointer cp, gchar *feedname) { 
	channelPtr	c = (channelPtr)cp;
	
	/* on new dialog the channel ptr is unknown... */
	if(NULL == c)
		c = new_cp;

	setFeedTitleInConfig(c->feedkey, feedname);	/* update in gconf */
	g_free(c->usertitle);				/* update local copy */
	c->usertitle = g_strdup(feedname);
	showFeed(c); 	/* refresh model */
}

void setFeedURL(gpointer cp, gchar *feedurl) {
	channelPtr	c = (channelPtr)cp;
	
	/* on new dialog the channel ptr is unknown... */
	if(NULL == c)
		c = new_cp;

	setFeedURLInConfig(c->feedkey, feedurl); 	/* update in gconf */
	g_free(c->source);				/* update local copy */
	c->link = g_strdup(feedurl);
	showFeed(c); 	/* refresh model */
}

/* just some encapsulation */
GHashTable * getFeedNSHandler(gpointer cp) { return nslist; }
gchar * getFeedSource(gpointer cp) { return ((channelPtr)cp)->source; }
gchar * getFeedURL(gpointer cp) { return ((channelPtr)cp)->link; }
gchar * getFeedImage(gpointer cp) { return ((channelPtr)cp)->image; }
gchar * getFeedTitle(gpointer cp) { return (NULL == ((channelPtr)cp)->usertitle)?((channelPtr)cp)->title:((channelPtr)cp)->usertitle; }
gchar * getFeedDescription(gpointer cp) { return ((channelPtr)cp)->description; }

gchar * getDefaultFeedTitle(gpointer cp) { 

	if(NULL == cp) 
		return new_cp->title;
	else
		return ((channelPtr)cp)->title; 
}

gboolean getReadStatus(gpointer ip) { return ((itemPtr)ip)->read; }
gchar * getItemTitle(gpointer ip) { return ((itemPtr)ip)->title; }
gchar *	getItemLink(gpointer ip) { return ((itemPtr)ip)->link; }
gchar * getItemContent(gpointer ip) { return ((itemPtr)ip)->description; }

void clearItems() {
	/* clear item list */
	gtk_tree_store_clear(GTK_TREE_STORE(itemstore));
}

void loadItems(gpointer cp) {
	GtkTreeIter	iter;
	itemPtr		item;
	
	clearItems();

	/* hmm... maybe we should store the parsed data as GtkTreeStores 
	   and exchange them ? */

	item = ((channelPtr)cp)->items;	
	while(NULL != item) {	
		gtk_tree_store_append(itemstore, &iter, NULL);
		gtk_tree_store_set(itemstore, &iter,
	     		   	IS_TITLE, item->title,
				IS_STATE, (item->read)?readIcon:unreadIcon,
				IS_PTR, (gpointer)item,
				IS_TIME, item->time,
				-1);	
		
		item = item->next;
	}
}

void markItemAsRead(gpointer ip, GtkTreeIter *iter) {

	((itemPtr)ip)->read = TRUE;

	gtk_tree_store_set(itemstore, iter,
			IS_STATE, readIcon,
			-1);		
	
}

GtkTreeStore * getItemStore(void) {

	if(NULL == itemstore) {
		/* set up a store of these attributes: 
			- item title
			- item state (read/unread)		
			- pointer to item data
			- a string containing the receival time

		 */
		itemstore = gtk_tree_store_new(4, G_TYPE_STRING, 
						  GDK_TYPE_PIXBUF, 
						  G_TYPE_POINTER, 
						  G_TYPE_STRING);
	}
	
	return itemstore;
}

GtkTreeStore * getFeedStore(void) {

	if(NULL == feedstore) {
		/* set up a store of these attributes: 
			- feed title
			- feed url
			- feed state icon (not available/available)
			- pointer to feed data
			- feed key in gconf
		 */
		feedstore = gtk_tree_store_new(5, G_TYPE_STRING, 
						  G_TYPE_STRING, 
						  GDK_TYPE_PIXBUF, 
						  G_TYPE_POINTER, 
						  G_TYPE_STRING);
	}
	
	return feedstore;
}

/* this function can be used to update any of the values by specifying
   the feedkey as first parameter */
void setInFeedList(gchar *feedkey, gchar * title, gchar *url, GdkPixbuf *state, channelPtr cp) {
	GtkTreeIter	iter;
	gboolean	valid, found = 0;	
	gchar		*tmp_title = NULL;
	gchar		*tmp_url = NULL;
	GdkPixbuf	*tmp_state = NULL;
	channelPtr	tmp_cp;

	valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(feedstore), &iter);
	while(valid) {
		gtk_tree_model_get(GTK_TREE_MODEL(feedstore), &iter,
				FS_TITLE, &tmp_title,
				FS_URL, &tmp_url,
				FS_STATE, &tmp_state,
				FS_CP, &tmp_cp,
		  	      -1);

		if((NULL != tmp_cp->feedkey) && 
		   (0 == strcmp(tmp_cp->feedkey, ((channelPtr)cp)->feedkey)))
			found = 1;		

		if(found) {
			/* insert changed row contents */
			gtk_tree_store_set(feedstore, &iter,
					   FS_TITLE, title,
					   FS_URL, url,
					   FS_STATE, state,
					   FS_CP, (gpointer) cp,
					  -1);
		}
		
		g_free(tmp_title);
		g_free(tmp_url);
		
		if(found)
			return;
		
		valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(feedstore), &iter);
	}

	/* if we come here, this is a not yet displayed feed */
	addToFeedList(cp);
}

// FIXME: how to add feed url that could not be retrieved???
void addToFeedList(channelPtr cp) {
	GtkTreeIter	iter;	// FIXME

	gtk_tree_store_append(feedstore, &iter, myfeedgroup);
	gtk_tree_store_set(feedstore, &iter,
			   FS_TITLE, (cp->usertitle)?cp->usertitle:cp->title,
			   FS_URL, cp->source,
			   FS_STATE, (cp->available)?availableIcon:unavailableIcon,
			   FS_CP, (gpointer) cp,
			   -1);	
}

/* feed list sorting functions */

void moveFeedPosition(gpointer cp, gint mode) {
	GtkTreeIter		iter;
	GSList			*keylist;
	GConfValue		*element;
	const char		*feedkey;
	channelPtr		c;
	
	switch(mode) {
		case 0:
			moveUpFeedPositionInConfig(((channelPtr)cp)->feedkey);
			break;
		case 1:
			moveDownFeedPositionInConfig(((channelPtr)cp)->feedkey);
			break;
		case 2:
			sortFeedKeyList();
			break;
		default:
			g_error(_("invalid feed list reorder move\n"));
			break;
	}

	gtk_tree_store_clear(GTK_TREE_STORE(feedstore));	

	keylist = getFeedKeyList();
	
	while(NULL != keylist) {
		element = keylist->data;	// FIXME: remove gconf dependency (convert list to normal list)
		feedkey = gconf_value_get_string(element);
		
		c = (channelPtr)g_hash_table_lookup(feeds, feedkey);
		
		addToFeedList(c);
		
		keylist = g_slist_next(keylist);	
	}
}

void moveUpFeedPosition(gpointer cp) { moveFeedPosition(cp, 0); }

void moveDownFeedPosition(gpointer cp) { moveFeedPosition(cp, 1); }

void sortFeeds(void) { moveFeedPosition(NULL, 2); }

	
/*	if(NULL == format)
		format = g_strdup("%H:%M %d.%m.");*/
