/*
   Dublin Core support
   
   Copyright (C) 2003 Lars Lindner <lars.lindner@gmx.net>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include <libgtkhtml/gtkhtml.h>
#include <time.h>
#include "ns_dc.h"
#include "conf.h"

#define	TIMESTRLEN	256

#define TABLE_START	"<table style=\"width:100%;border-width:1px;border-top-style:solid;border-color:#D0D0D0;\">"
#define FIRSTTD		"<tr style=\"border-width:0;border-bottom-width:1px;border-style:dashed;border-color:#D0D0D0;\"><td><span style=\"font-size:x-small;color:#C0C0C0\">"
#define NEXTTD		"</span></td><td><span style=\"font-size:x-small;color:#C0C0C0\">"
#define LASTTD		"</span></td></tr>"
#define TABLE_END	"</table>"

#define HTML_WRITE(doc, tags)	html_document_write_stream(doc, tags, strlen(tags));

static gchar ns_dc_prefix[] = "dc";

/* some prototypes */
void ns_dc_parseChannelTag(channelPtr cp, xmlDocPtr doc, xmlNodePtr cur);
void ns_dc_parseItemTag(itemPtr ip,xmlDocPtr doc, xmlNodePtr cur);

//gchar * ns_dc_doItemHeaderOutput(gpointer obj, gpointer htmlStream);
gchar * ns_dc_doItemFooterOutput(gpointer obj, gpointer htmlStream);
//gchar *	ns_dc_doChannelHeaderOutput(gpointer obj, gpointer htmlStream);
gchar *	ns_dc_doChannelFooterOutput(gpointer obj, gpointer htmlStream);

/* a tag list from http://web.resource.org/rss/1.0/modules/dc/

-------------------------------------------------------
 <channel>, <item>, <image>, and <textinput> Elements:

    * <dc:title> ( #PCDATA )
    * <dc:creator> ( #PCDATA )
    * <dc:subject> ( #PCDATA )
    * <dc:description> ( #PCDATA )
    * <dc:publisher> ( #PCDATA )
    * <dc:contributor> ( #PCDATA )
    * <dc:date> ( #PCDATA ) [W3CDTF]
    * <dc:type> ( #PCDATA )
    * <dc:format> ( #PCDATA )
    * <dc:identifier> ( #PCDATA )
    * <dc:source> ( #PCDATA )
    * <dc:language> ( #PCDATA )
    * <dc:relation> ( #PCDATA )
    * <dc:coverage> ( #PCDATA )
    * <dc:rights> ( #PCDATA )
-------------------------------------------------------

Most feeds don't use title, subject and description so
we ignore them. The most important is the data tag for
correct update handling. When reading this tag we 
adjust the items date. 
Any other infos are simply displayed in channel info
or item view inside the header or footer of display.

*/

static gchar * taglist[] = {	/*"title",*/
				"creator",
				/*"subject",*/
				/*"description",*/
				"publisher",
				"contributor",				
				/*"date", ... handled differently */
				"type",
				"format",
				"identifier",
				"source",
				"language",
				"coverage",
				"rights",
				NULL
			   };
			   
/* the HTML stream the output handler write to */			   
static HtmlDocument	*doc;

gchar * ns_dc_getNSPrefix(void) { return ns_dc_prefix; }

nsHandler *ns_dc_getNameSpaceHandler(void) {
	nsHandler 	*nsh;
	
	if(NULL != (nsh = (nsHandler *)g_malloc(sizeof(nsHandler)))) {
		nsh->parseChannelTag		= ns_dc_parseChannelTag;
		nsh->parseItemTag		= ns_dc_parseItemTag;
		nsh->doChannelHeaderOutput	= NULL;
		nsh->doChannelFooterOutput	= ns_dc_doChannelFooterOutput;
		nsh->doItemHeaderOutput		= NULL;
		nsh->doItemFooterOutput		= ns_dc_doItemFooterOutput;		
	}

	return nsh;
}

static void ns_dc_addInfoStruct(GHashTable *nslist, gchar *tagname, gchar *tagvalue) {
	GHashTable	*nsvalues;
	
	g_assert(nslist != NULL);

	if(tagvalue == NULL)
		return;

	if(NULL == (nsvalues = (GHashTable *)g_hash_table_lookup(nslist, ns_dc_prefix))) {
		nsvalues = g_hash_table_new(g_str_hash, g_str_equal);
		g_hash_table_insert(nslist, (gpointer)ns_dc_prefix, (gpointer)nsvalues);
	}
	g_hash_table_insert(nsvalues, (gpointer)tagname, (gpointer)tagvalue);
}

void ns_dc_parseChannelTag(channelPtr cp, xmlDocPtr doc, xmlNodePtr cur) {
	int 	i;
	
	while (cur != NULL) {
		/* compare with each possible tag name */
		for(i = 0; taglist[i] != NULL; i++) {
	
			if(!xmlStrcmp((const xmlChar *)taglist[i], cur->name)) {
				ns_dc_addInfoStruct(cp->nsinfos, taglist[i],  xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
			}
		}
		
		cur = cur->next;
	}
}

void ns_dc_parseItemTag(itemPtr ip,xmlDocPtr doc, xmlNodePtr cur) {
	int 		i;
	gchar		*date;
	gchar		*timestr;
	gchar		*timeformat;
	struct tm	tm;
	time_t		t;
	
	while (cur != NULL) {
		/* special handling for dates */
		if(!xmlStrcmp((const xmlChar *)"date", cur->name)) {
			date = g_strdup(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
			memset(&tm, 0, sizeof(struct tm));
			
			/* we expect something like "2003-08-07T15:28:19" and ignore all 
			   timezone and second fraction specific e.g. in  
			   "2003-08-07T15:28:19.4+00:00" as defined in ISO 8601
			 */
			if(strlen(date) >= 19) {
				strptime((char *)date, "%Y-%m-%dT%T", &tm);				
				g_free(date);
				g_free(ip->time);

				if(NULL != (timestr = (gchar *)g_malloc(TIMESTRLEN+1))) {
					if(NULL != (timeformat = getStringConfValue(TIME_FORMAT))) {
						// FIXME: not thread safe and no error checking
						t = mktime(&tm);
						strftime(timestr, TIMESTRLEN, (char *)timeformat, &tm);
						ip->time=timestr;						
						g_free(timeformat);
					}
				}

			} else {
				g_print("Invalid date format! Ignoring <dc:date> information!\n");
			}
			
			cur = cur->next;		
			continue;
		}
	
		/* compare with each possible tag name */
		for(i = 0; taglist[i] != NULL; i++) {
	
			if(!xmlStrcmp((const xmlChar *)taglist[i], cur->name)) {
				ns_dc_addInfoStruct(ip->nsinfos, taglist[i],  xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
			}
		}
		
		cur = cur->next;
	}	
}

void ns_dc_output(gpointer key, gpointer value, gpointer userdata) {

	HTML_WRITE(doc, FIRSTTD);
	HTML_WRITE(doc, (gchar *)key);
	HTML_WRITE(doc, NEXTTD);
	HTML_WRITE(doc, (gchar *)value);
	HTML_WRITE(doc, LASTTD);
}

void ns_dc_doFooterOutput(GHashTable *nsinfos) {
	GHashTable	*nsvalues;
	
	/* we print all channel infos as a (key,value) table */
	if(NULL != (nsvalues = g_hash_table_lookup(nsinfos, (gpointer)ns_dc_prefix))) {
		HTML_WRITE(doc, TABLE_START);
		g_hash_table_foreach(nsvalues, ns_dc_output, NULL);
		HTML_WRITE(doc, TABLE_END);			
	}
}

gchar * ns_dc_doItemFooterOutput(gpointer obj, gpointer htmlStream) {

	doc = (HtmlDocument *)htmlStream;
	
	if((obj != NULL) && (doc != NULL)) {
		ns_dc_doFooterOutput(((itemPtr)obj)->nsinfos);
	}
}


gchar *	ns_dc_doChannelFooterOutput(gpointer obj, gpointer htmlStream) {
	doc = (HtmlDocument *)htmlStream;
	
	if((obj != NULL) && (doc != NULL)) {
		ns_dc_doFooterOutput(((channelPtr)obj)->nsinfos);
	}
}
